﻿Imports Oracle.DataAccess.Client
Imports Oracle.DataAccess.Types
'Imports HisKyoyu.clsDefine
'Imports HisKyoyu.clsFUNCTION


Public Class clsDBIF
    '******************************************************************************
    '***
    '*** <EXP>      DB操作共通クラス
    '*** <Create>   S.I    2010/09/08 ODP.NET用に修正
    '*** <Method一覧>
    '*** 　１．Execute - 指定したSQLを実行する
    '*** 　     [In]
    '*** 　        ByVal SQL As String          - 実行するSQL文
    '*** 　     [Ret]
    '*** 　        Execute As Int 32            - 影響を受けたレコード件数
    '***
    '*** 　２．GetValue - 指定したSQLを実行して、結果の一行目・一列目の値を返す
    '*** 　     [In]
    '*** 　        ByVal SQL As String          - 実行するSQL文
    '*** 　     [Ret]
    '*** 　        GetValue As Object           - 一行目・一列目の値
    '***
    '*** 　３．GetRecordset - 指定したSQLの結果セットを取得
    '*** 　     [In]
    '*** 　        ByVal SQL As String          - 実行するSQL文
    '*** 　     [Ret]
    '*** 　        GetRecordset As clsRecordset - 結果の仮想表
    '***
    '*** 　４．BeginTrans - Transactionを開始
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        BeginTrans As Boolean        - 開始できたかできないか
    '***
    '*** 　５．Rollback - Transaction内のデータ操作を元に戻す
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        Rollback As Boolean          - 元に戻せたかどうか
    '***
    '*** 　６．Commit - Transaction内のデータ操作を確定する
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        Commit As Boolean            - 確定できたかできないか
    '***
    '*** 　７．Open  - DBとの接続を開始する
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        Open As Boolean              - 接続できたかできないか
    '***
    '*** 　８．GetSqlCommand  - 当接続で利用しているSQLCommandを返す
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        GetSqlCommand As SQLCommand             - 当接続で利用しているSQLCommand
    '***
    '*** 　９．Close  - DBとの接続を切る
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        Close As Boolean             - 接続を切れたかどうか
    '***
    '*** 　10．Cnn  - 当接続で使用しているOracleConnectionを返す
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        pobjCn As OracleConnection             - 当接続で使用しているOracleConnection
    '***
    '******************************************************************************

    '/// ローカル変数
    'Private pobjCmd As New OracleCommand
    'Private pobjCn As New OracleConnection
    'Private pobjTrn As OracleTransaction
    'Private pobjDA As New OracleDataAdapter
    'Private pobjDS As New DataSet
    'Private pstrCn As String = ""
    'Private pblnTrn As Boolean = False

    Private pobjCn As New OleDb.OleDbConnection
    Private pobjCmd As New OleDb.OleDbCommand
    Private pobjTrn As OleDb.OleDbTransaction
    Private pobjDA As New OleDb.OleDbDataAdapter
    Private pobjDS As New DataSet
    Private pstrCn As String = ""
    Private pblnTrn As Boolean = False

#Region "クラス初期化"
    Public Sub New()

        Dim dbNm As String

        ''--- ※他のシステムで利用する場合は、この部分をカスタマイズしてください。---
        ''/// DB接続情報の確認
        'If gstrDBSV = "" Then
        '    '/// 読み込まれていなさそうな場合は、再度初期設定XMLより情報取得
        '    Call clsFUNCTION.GF_ReadConfig()
        'End If


        '/// 接続文字列作成
        'pstrCn = ""
        'pstrCn &= "user id=" & My.Resources.gstrDBID & ";"
        'pstrCn &= "password=" & My.Resources.gstrDBPW & ";"
        'pstrCn &= "data source=" & My.Resources.gstrDBNM & ";"
        ''----------------------------------------------------------------------------

        'Setteingから取得できなかった場合はKoyuZaisan.resourceから取得
        dbNm = IIf(My.Settings.dbNm = "", My.Resources.gstrDBNM, My.Settings.dbNm)

        '/// 接続文字列作成
        'If UID = "" Then
        pstrCn = "Provider=OraOLEDB.Oracle.1;" & _
                 "Persist Security Info=True;" & _
                 "Data Source=" & dbNm & ";" & _
                 "User ID=" & My.Resources.gstrDBID & ";" & _
                 "Password=" & My.Resources.gstrDBPW & ";"
        'Else
        '    pstrCn = "Provider=OraOLEDB.Oracle.1;" & _
        '             "Persist Security Info=True;" & _
        '             "Data Source=" & DBNM & ";" & _
        '             "User ID=" & UID & ";" & _
        '             "Password=" & PWD & ";"
        'End If


        '/// 接続文字列を設定
        pobjCn.ConnectionString = pstrCn

        '/// SQLCommandとSQLConnctionの関連付け
        pobjCmd.Connection = pobjCn

        '/// SQLCommandのタイムアウト設定（※無制限に設定します）
        pobjCmd.CommandTimeout = 0

    End Sub
#End Region

#Region "指定したSQLを実行する"
    Public Function Execute(ByVal SQL As String) As Int32
        pobjCmd.CommandText = SQL
        Return pobjCmd.ExecuteNonQuery
    End Function
#End Region

#Region "指定したSQLを実行して、結果の一行目・一列目の値を返す"
    Public Function GetValue(ByVal SQL As String) As Object
        pobjCmd.CommandText = SQL
        Return pobjCmd.ExecuteScalar
    End Function
#End Region

#Region "指定したSQLの結果セットを取得"
    Public Function GetRecordset(ByVal SQL As String) As clsRecordset
        Static i As Int32
        i = i + 1
        pobjCmd.CommandText = SQL
        pobjDA.SelectCommand = pobjCmd
        pobjDA.Fill(pobjDS, "TMP" & i)
        Dim objRS As clsRecordset = New clsRecordset(pobjDS.Tables("TMP" & i))
        Return objRS
    End Function
#End Region

#Region "Transactionを開始"
    Public Function BeginTrans() As Boolean
        pobjTrn = pobjCn.BeginTransaction()
        pobjCmd.Transaction = pobjTrn
        pblnTrn = True
        Return True

    End Function
#End Region

#Region "Transaction内のデータ操作を元に戻す"
    Public Function Rollback() As Boolean
        If pblnTrn = True Then
            pobjTrn.Rollback()
            pobjTrn.Dispose()
            pblnTrn = False
        End If
        Return True
    End Function
#End Region

#Region "Transaction内のデータ操作を確定する"
    Public Function Commit() As Boolean
        pobjTrn.Commit()
        pobjTrn.Dispose()
        pblnTrn = False
        Return True
    End Function
#End Region

#Region "DBとの接続を開始する"
    Public Function Open() As Boolean
        pobjCn.Open()
        Return True
    End Function
#End Region

#Region "DBとの接続を切る"
    Public Function Close() As Boolean
        If pblnTrn = True Then
            pobjTrn.Rollback()
            pobjTrn.Dispose()
            pblnTrn = False
        End If
        pobjCn.Close()
        Return True

    End Function
#End Region

#Region "当接続で利用しているSQLCommandを返す"
    '    Public Function GetSqlCommand() As OracleCommand
    '        Return pobjCmd
    '    End Function
    '#End Region

    '#Region "当接続で使用しているOracleConnectionを返す"
    '    Public Function Cnn() As OracleConnection
    '        Return pobjCn
    '    End Function
    '#End Region
#End Region

#Region "当接続で利用しているSQLCommandを返す"
    Public Function GetSqlCommand() As OleDb.OleDbCommand
        Return pobjCmd
    End Function
#End Region

#Region "当接続で使用しているOracleConnectionを返す"
    Public Function Cnn() As OleDb.OleDbConnection
        Return pobjCn
    End Function
#End Region

#Region "クラス解放"
    Public Sub Dispose()
        Call Finalize()
    End Sub
    Protected Overrides Sub Finalize()
        '/// 変数開放
        On Error Resume Next
        pobjCn.Dispose()
        pobjCmd.Dispose()
        If pblnTrn = True Then pobjTrn.Dispose()
        pobjDA.Dispose()
        pobjDS.Dispose()
        pstrCn = Nothing
        pblnTrn = Nothing
        On Error GoTo 0
        MyBase.Finalize()
    End Sub
#End Region

End Class

