﻿Imports Oracle.DataAccess.Client
Imports Oracle.DataAccess.Types

Public Class clsRecordset
    '******************************************************************************
    '***
    '*** <EXP>      共通操作用レコードセットクラス（DB操作共通クラスからの派生クラス）
    '*** <CREATE>   2010/09/13 By S.I 
    '***
    '*** <Property一覧>
    '*** 　１．EOF - EOFか判定
    '*** 　２．RecordCount - レコード件数
    '*** 　３．RecordIndex - 現在の行（０から数えてます）
    '*** 　４．FieldCount  - フィールド数
    '***
    '*** <Method一覧>
    '*** 　１．MoveFirst - 先頭行に移動
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        無し
    '***
    '*** 　２．MoveLast  - 最終行に移動
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        無し
    '***
    '*** 　３．MoveN  - 指定行に移動
    '*** 　     [In]
    '*** 　        ByVal Index As Int32         - 移動する行数（０からはじまります）
    '*** 　     [Ret]
    '*** 　        無し
    '***
    '*** 　４．MoveNext  - 次の行に移動
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        無し
    '***
    '*** 　５．MovePrevious  - 前の行に移動
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        無し
    '***
    '*** 　６．Value - 現在の行の指定フィールドの値を返す
    '*** 　     [In]
    '*** 　        ByVale FieldNm As String     - フィールド名
    '*** 　        Or
    '*** 　        ByVale FieldIndex As Int32   - フィールドインデックス（0からのインデックス）
    '*** 　     [Ret]
    '*** 　        Value As Object              - 値
    '***
    '*** 　７．FiledName - 現在の行の指定フィールドの値を返す
    '*** 　     [In]
    '*** 　        ByVale FieldIndex As Int32   - 指定したインデックス番目フィールドの名を返す（0からのインデックス）
    '*** 　     [Ret]
    '*** 　        FiledName As String          - フィールド名
    '***
    '******************************************************************************

    '/// ローカル変数
    Private pobjTable As DataTable

    '/// グローバル変数
    Public EOF As Boolean = False
    Public RecordCount As Int32 = 0
    Public RecordIndex As Int32 = 0
    Public FieldCount As Int32 = 0

#Region "クラス初期化"
    Public Sub New(ByVal objTable As DataTable)
        pobjTable = objTable
        RecordCount = pobjTable.Rows.Count
        RecordIndex = 0
        FieldCount = pobjTable.Columns.Count
        If pobjTable.Rows.Count = 0 Then
            EOF = True
        End If
    End Sub
#End Region

#Region "先頭行に移動"
    Public Sub MoveFirst()
        RecordIndex = 0
        If RecordCount = 0 Then
            EOF = True
        Else
            EOF = False
        End If
    End Sub
#End Region

#Region "最終行に移動"
    Public Sub MoveLast()
        RecordIndex = RecordCount - 1
        If RecordCount = 0 Then
            EOF = True
        Else
            EOF = False
        End If
    End Sub
#End Region

#Region "指定行に移動"
    Public Sub MoveN(ByVal Index As Int32)
        RecordIndex = Index
        If RecordCount <= Index Or RecordCount = 0 Then
            EOF = True
        Else
            EOF = False
        End If
    End Sub
#End Region

#Region "次の行に移動"
    Public Sub MoveNext()
        RecordIndex = RecordIndex + 1
        If RecordCount <= RecordIndex Or RecordCount = 0 Then
            EOF = True
        Else
            EOF = False
        End If
    End Sub
#End Region

#Region "前の行に移動"
    Public Sub MovePrvious()
        RecordIndex = RecordIndex - 1
        If RecordIndex < 0 Then RecordIndex = 0
        If RecordCount = 0 Then
            EOF = True
        End If
    End Sub
#End Region

#Region "現在の行の指定フィールドの値を返す"
    Public Function Value(ByVal FieldNm As String) As Object
        Return pobjTable.Rows(RecordIndex).Item(FieldNm)
    End Function
    Public Function Value(ByVal FieldIndex As Int32) As Object
        Return pobjTable.Rows(RecordIndex).Item(FieldIndex)
    End Function
#End Region

#Region "フィールド名を返す"
    Public Function FieldNm(ByVal FieldIndex As Int32) As String
        Return pobjTable.Columns(FieldIndex).ColumnName
    End Function
#End Region

#Region "指定した条件に一致するレコードがあるか返す"
    Public Function Exist(ByVal strWhere As String) As Boolean
        Dim intRet As Int32
        intRet = pobjTable.Select(strWhere).Length
        If intRet = 0 Then
            Return False
        Else
            Return True
        End If
    End Function
#End Region

#Region "指定した条件に一致する最初レコードにカーソルを移動する"
    Public Function FindFirst(ByVal strWhere As String) As Boolean
        Dim intLoop As Int32
        Dim objRows() As DataRow

        objRows = pobjTable.Select(strWhere)
        If objRows.Length = 0 Then
            Return False
        Else
            For intLoop = 0 To pobjTable.Rows.Count - 1
                If objRows(0).Equals(pobjTable.Rows(intLoop)) Then
                    RecordIndex = intLoop
                    Exit For
                End If
            Next
            Return True
        End If
    End Function
#End Region

#Region "指定した条件に一致する次のレコードにカーソルを移動する"
    Public Function FindNext(ByVal strWhere As String) As Boolean
        Dim intLoop As Int32
        Dim objRows() As DataRow

        objRows = pobjTable.Select(strWhere)
        If objRows.Length = 0 Then
            Return False
        Else
            For intLoop = RecordIndex + 1 To pobjTable.Rows.Count - 1
                If objRows(0).Equals(pobjTable.Rows(intLoop)) Then
                    RecordIndex = intLoop
                    Exit For
                End If
            Next
            Return True
        End If
    End Function
#End Region

#Region "指定した条件に一致する最後レコードにカーソルを移動する"
    Public Function FindLast(ByVal strWhere As String) As Boolean
        Dim intLoop As Int32
        Dim objRows() As DataRow

        objRows = pobjTable.Select(strWhere)
        If objRows.Length = 0 Then
            Return False
        Else
            For intLoop = pobjTable.Rows.Count - 1 To 0 Step -1
                If objRows(0).Equals(pobjTable.Rows(intLoop)) Then
                    RecordIndex = intLoop
                    Exit For
                End If
            Next
            Return True
        End If
    End Function
#End Region

#Region "クラス開放"
    Public Sub Dispose()
        Call Finalize()
    End Sub
    Protected Overrides Sub Finalize()
        On Error Resume Next
        If IsNothing(pobjTable.DataSet) = False Then pobjTable.DataSet.Tables.Remove(pobjTable)
        pobjTable.Dispose()
        On Error GoTo 0
        MyBase.Finalize()
    End Sub
#End Region

#Region "当レコードセットの実体（DataTable）を取得"
    Public Function GetDataTable() As DataTable
        Return pobjTable.Copy
    End Function
#End Region

End Class
