﻿Imports System.Data
Imports System.Data.SqlClient
Imports AnzenSuishin.clsVariable

Public Class clsDBIF
    '******************************************************************************
    '***
    '*** <EXP>      DB操作共通クラス
    '*** <Create>   M.Yano 2007/08/15
    '*** <Method一覧>
    '*** 　１．Execute - 指定したSQLを実行する
    '*** 　     [In]
    '*** 　        ByVal SQL As String          - 実行するSQL文
    '*** 　     [Ret]
    '*** 　        Execute As Int 32            - 影響を受けたレコード件数
    '***
    '*** 　２．GetValue - 指定したSQLを実行して、結果の一行目・一列目の値を返す
    '*** 　     [In]
    '*** 　        ByVal SQL As String          - 実行するSQL文
    '*** 　     [Ret]
    '*** 　        GetValue As Object           - 一行目・一列目の値
    '***
    '*** 　３．GetRecordset - 指定したSQLの結果セットを取得
    '*** 　     [In]
    '*** 　        ByVal SQL As String          - 実行するSQL文
    '*** 　     [Ret]
    '*** 　        GetRecordset As clsRecordset - 結果の仮想表
    '***
    '*** 　４．BeginTrans - Transactionを開始
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        BeginTrans As Boolean        - 開始できたかできないか
    '***
    '*** 　５．Rollback - Transaction内のデータ操作を元に戻す
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        Rollback As Boolean          - 元に戻せたかどうか
    '***
    '*** 　６．Commit - Transaction内のデータ操作を確定する
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        Commit As Boolean            - 確定できたかできないか
    '***
    '*** 　７．Open  - DBとの接続を開始する
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        Open As Boolean              - 接続できたかできないか
    '***
    '*** 　８．GetSqlCommand  - 当接続で利用しているSQLCommandを返す
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        GetSqlCommand As SQLCommand             - 当接続で利用しているSQLCommand
    '***
    '*** 　９．CreateTableFromDataTable  - DataTableを元にテーブルを作成
    '*** 　     [In]
    '*** 　        objDT As DataTable               - 元となるDataTable
    '*** 　        strTableName As String           - 作成するテーブル名
    '*** 　        BeforeDelete As Boolean = False  - 作成前にテーブルを削除するか
    '*** 　     [Ret]
    '*** 　        無し
    '***
    '*** １０．Close  - DBとの接続を切る
    '*** 　     [In]
    '*** 　        無し
    '*** 　     [Ret]
    '*** 　        Close As Boolean             - 接続を切れたかどうか
    '***
    '*** Copyright (C) 2007 矢野 誠
    '***
    '******************************************************************************

    '/// ローカル変数
    Private pobjCn As New SqlConnection
    Private pobjCmd As New SqlCommand
    Private pobjTrn As SqlTransaction
    Private pobjDA As New SqlDataAdapter
    Private pobjDS As New DataSet
    Private pstrCn As String = ""
    Private pblnTrn As Boolean = False

#Region "クラス初期化"
    Public Sub New()

        '--- ※他のシステムで利用する場合は、この部分をカスタマイズしてください。---

        '/// DB接続情報の確認
        If gstrDBSV = "" Then
            '/// 読み込まれていなさそうな場合は、再度初期設定XMLより情報取得
            Call clsFunction.GF_SetDBInformation()
        End If

        '/// 接続文字列作成
        pstrCn = ""
        pstrCn &= "Persist Security Info=True;"
        pstrCn &= "SERVER=" & gstrDBSV & ";"
        pstrCn &= "DATABASE=" & gstrDBNM & ";"
        pstrCn &= "UID=" & gstrDBID & ";"
        pstrCn &= "PWD=" & gstrDBPW & ""
        '----------------------------------------------------------------------------

        '/// 接続文字列を設定
        pobjCn.ConnectionString() = pstrCn

        '/// SQLCommandとSQLConnctionの関連付け
        pobjCmd.Connection() = pobjCn

        '/// SQLCommandのタイムアウト設定（※無制限に設定します）
        pobjCmd.CommandTimeout = 0

    End Sub
    Public Sub New(ByRef objSQLCommand As SqlCommand)

        '--- objSQLCommandで渡された接続でクラスを初期化

        ''/// 接続文字列を設定
        'pobjCn = objSQLCommand

        '/// SQLCommandとSQLConnctionの関連付け
        pobjCmd = objSQLCommand

    End Sub
#End Region

#Region "指定したSQLを実行する"
    Public Function Execute(ByVal SQL As String) As Int32
        pobjCmd.CommandText = SQL
        Return pobjCmd.ExecuteNonQuery
    End Function
#End Region

#Region "指定したSQLを実行して、結果の一行目・一列目の値を返す"
    Public Function GetValue(ByVal SQL As String) As Object
        pobjCmd.CommandText = SQL
        Return pobjCmd.ExecuteScalar
    End Function
#End Region

#Region "指定したSQLの結果セットを取得"
    Public Function GetRecordset(ByVal SQL As String) As clsRecordset
        'Dim objTbl As DataTable
        Static i As Int32
        i = i + 1
        pobjCmd.CommandText = SQL
        pobjDA.SelectCommand = pobjCmd
        pobjDA.Fill(pobjDS, "TMP" & i)
        Dim objRS As clsRecordset = New clsRecordset(pobjDS.Tables("TMP" & i))
        Return objRS
    End Function
#End Region

#Region "Transactionを開始"
    Public Function BeginTrans() As Boolean
        pobjTrn = pobjCn.BeginTransaction()
        pobjCmd.Transaction = pobjTrn
        pblnTrn = True
        Return True
    End Function
#End Region

#Region "Transaction内のデータ操作を元に戻す"
    Public Function Rollback() As Boolean
        If pblnTrn = True Then
            pobjTrn.Rollback()
            pobjTrn.Dispose()
            pblnTrn = False
        End If
        Return True
    End Function
#End Region

#Region "Transaction内のデータ操作を確定する"
    Public Function Commit() As Boolean
        pobjTrn.Commit()
        pobjTrn.Dispose()
        pblnTrn = False
        Return True
    End Function
#End Region

#Region "DBとの接続を開始する"
    Public Function Open() As Boolean
        pobjCn.Open()
        Return True
    End Function
#End Region

#Region "DBとの接続を切る"
    Public Function Close() As Boolean
        If pblnTrn = True Then
            pobjTrn.Rollback()
            pobjTrn.Dispose()
            pblnTrn = False
        End If
        pobjCn.Close()
        Return True
    End Function
#End Region

#Region "当接続で利用しているSQLCommandを返す"
    Public Function GetSqlCommand() As SqlCommand
        Return pobjCmd
    End Function
#End Region

#Region "DataTableを元にテーブルを作成"
    Public Sub CreateTableFromDataTable(ByVal objDT As DataTable, ByVal strTableName As String, Optional ByVal BeforeDelete As Boolean = False)

        Dim strSQL As String
        Dim objCol As DataColumn
        Dim objRow As DataRow

        '/// 始めにテーブルを削除
        If BeforeDelete Then
            strSQL = ""
            strSQL &= "if exists (select * from dbo.sysobjects where id = object_id(N'" & strTableName & "') and OBJECTPROPERTY(id, N'IsUserTable') = 1)" & vbCrLf
            strSQL &= "drop table " & strTableName & "" & vbCrLf
            pobjCmd.CommandText = strSQL
            pobjCmd.ExecuteNonQuery()
        End If

        '/// テーブルCreate文を作成
        strSQL = ""
        strSQL &= "Create Table " & strTableName & " (" & vbCrLf

        For Each objCol In objDT.Columns
            Select Case objCol.DataType.ToString.ToUpper
                Case "System.Boolean".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [smallint], " & vbCrLf
                Case "System.Byte".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [smallint], " & vbCrLf
                Case "System.Char".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [char] (256), " & vbCrLf
                Case "System.DateTime".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [dateTime], " & vbCrLf
                Case "System.Decimal".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [varchar] (256), " & vbCrLf        '--- 対応しきれないのでvarchar（255）
                Case "System.Double".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [float], " & vbCrLf
                Case "System.Int16".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [smallint], " & vbCrLf
                Case "System.Int32".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [int], " & vbCrLf
                Case "System.Int64".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [bigint], " & vbCrLf
                Case "System.SByte".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [smallint], " & vbCrLf
                Case "System.Single".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [float], " & vbCrLf
                Case "System.String".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [varchar] (512), " & vbCrLf
                Case "System.TimeSpan".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [varchar] (256), " & vbCrLf        '--- 対応しきれないのでvarchar（255）
                Case "System.UInt16".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [int], " & vbCrLf
                Case "System.UInt32".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [bigint], " & vbCrLf
                Case "System.UInt64".ToUpper
                    strSQL &= "  [" & objCol.ColumnName & "] [decimal] (19,0), " & vbCrLf
                Case Else
                    strSQL &= "  [" & objCol.ColumnName & "] [varchar] (512), " & vbCrLf        '--- その他の型はvarchar（255）
            End Select
        Next

        strSQL = strSQL.Substring(0, strSQL.LastIndexOf(",")) & ")"
        pobjCmd.CommandText = strSQL
        pobjCmd.ExecuteNonQuery()

        '/// 作成したテーブルに行追加
        For Each objRow In objDT.Rows
            strSQL = ""
            strSQL &= "INSERT INTO " & strTableName & " " & vbCrLf
            strSQL &= "VALUES " & vbCrLf
            strSQL &= "  ( " & vbCrLf

            For Each objCol In objDT.Columns
                Select Case objCol.DataType.ToString.ToUpper
                    Case "System.Boolean".ToUpper
                        strSQL &= "    " & CType(objRow(objCol), Int16) & ", " & vbCrLf
                    Case "System.Byte".ToUpper
                        strSQL &= "    " & CType(objRow(objCol), Int16) & ", " & vbCrLf
                    Case "System.Char".ToUpper
                        strSQL &= "    '" & objRow(objCol) & "', " & vbCrLf
                    Case "System.DateTime".ToUpper
                        strSQL &= "    " & objRow(objCol).ToString("yyyy/MM/dd HH:mm:ss") & ", " & vbCrLf
                    Case "System.Decimal".ToUpper
                        strSQL &= "    '" & objRow(objCol).ToString & "', " & vbCrLf
                    Case "System.Double".ToUpper
                        strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                    Case "System.Int16".ToUpper
                        strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                    Case "System.Int32".ToUpper
                        strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                    Case "System.Int64".ToUpper
                        strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                    Case "System.SByte".ToUpper
                        strSQL &= "    " & CType(objRow(objCol), Int16) & ", " & vbCrLf
                    Case "System.Single".ToUpper
                        strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                    Case "System.String".ToUpper
                        strSQL &= "    '" & objRow(objCol).ToString & "', " & vbCrLf
                    Case "System.TimeSpan".ToUpper
                        strSQL &= "    '" & objRow(objCol).ToString & "', " & vbCrLf
                    Case "System.UInt16".ToUpper
                        strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                    Case "System.UInt32".ToUpper
                        strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                    Case "System.UInt64".ToUpper
                        strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                    Case Else
                        strSQL &= "    '" & objRow(objCol).ToString & "', " & vbCrLf
                End Select
            Next
            strSQL = strSQL.Substring(0, strSQL.LastIndexOf(",")) & ")"
            pobjCmd.CommandText = strSQL
            pobjCmd.ExecuteNonQuery()
        Next

    End Sub
#End Region

#Region "DataTableの内容をテーブルに追加"
    Public Sub InsertTableFromDataTable(ByVal objDT As DataTable, ByVal strTableName As String, Optional ByVal BeforeTruncate As Boolean = False)

        Dim strSQL As String
        Dim objCol As DataColumn
        Dim objRow As DataRow

        If BeforeTruncate = True Then
            strSQL = "TRUNCATE TABLE " & strTableName
            pobjCmd.CommandText = strSQL
            pobjCmd.ExecuteNonQuery()
        End If

        '/// 作成したテーブルに行追加
        For Each objRow In objDT.Rows
            strSQL = ""
            strSQL &= "INSERT INTO " & strTableName & " " & vbCrLf
            strSQL &= "  ( " & vbCrLf

            For Each objCol In objDT.Columns
                If IsDBNull(objRow(objCol)) = False Then
                    strSQL &= "    [" & objCol.ColumnName & "], " & vbCrLf
                End If
            Next
            strSQL = strSQL.Substring(0, strSQL.LastIndexOf(",")) & vbCrLf

            strSQL &= "  ) " & vbCrLf
            strSQL &= "VALUES " & vbCrLf
            strSQL &= "  ( " & vbCrLf

            For Each objCol In objDT.Columns
                If IsDBNull(objRow(objCol)) = False Then
                    Select Case objCol.DataType.ToString.ToUpper
                        Case "System.Boolean".ToUpper
                            strSQL &= "    " & CType(objRow(objCol), Int16) & ", " & vbCrLf
                        Case "System.Byte".ToUpper
                            strSQL &= "    " & CType(objRow(objCol), Int16) & ", " & vbCrLf
                        Case "System.Char".ToUpper
                            strSQL &= "    '" & objRow(objCol) & "', " & vbCrLf
                        Case "System.DateTime".ToUpper
                            strSQL &= "    " & objRow(objCol).ToString("yyyy/MM/dd HH:mm:ss") & ", " & vbCrLf
                        Case "System.Decimal".ToUpper
                            strSQL &= "    '" & objRow(objCol).ToString & "', " & vbCrLf
                        Case "System.Double".ToUpper
                            strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                        Case "System.Int16".ToUpper
                            strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                        Case "System.Int32".ToUpper
                            strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                        Case "System.Int64".ToUpper
                            strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                        Case "System.SByte".ToUpper
                            strSQL &= "    " & CType(objRow(objCol), Int16) & ", " & vbCrLf
                        Case "System.Single".ToUpper
                            strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                        Case "System.String".ToUpper
                            strSQL &= "    '" & objRow(objCol).ToString & "', " & vbCrLf
                        Case "System.TimeSpan".ToUpper
                            strSQL &= "    '" & objRow(objCol).ToString & "', " & vbCrLf
                        Case "System.UInt16".ToUpper
                            strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                        Case "System.UInt32".ToUpper
                            strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                        Case "System.UInt64".ToUpper
                            strSQL &= "    " & objRow(objCol) & ", " & vbCrLf
                        Case Else
                            strSQL &= "    '" & objRow(objCol).ToString & "', " & vbCrLf
                    End Select
                End If
            Next
            strSQL = strSQL.Substring(0, strSQL.LastIndexOf(",")) & ")"
            pobjCmd.CommandText = strSQL
            pobjCmd.ExecuteNonQuery()
        Next

    End Sub
#End Region

#Region "クラス解放"
    Public Sub Dispose()
        Call Finalize()
    End Sub
    Protected Overrides Sub Finalize()
        '/// 変数開放
        On Error Resume Next
        pobjCn.Dispose()
        pobjCmd.Dispose()
        If pblnTrn = True Then pobjTrn.Dispose()
        pobjDA.Dispose()
        pobjDS.Dispose()
        pstrCn = Nothing
        pblnTrn = Nothing
        On Error GoTo 0
        MyBase.Finalize()
    End Sub
#End Region

End Class
