﻿Imports System.ComponentModel
Imports System.Windows.Forms

''' <summary>
''' 拡張コンボボックス
''' </summary>
''' <remarks></remarks>
Public Class ComboBoxEx
    Inherits System.Windows.Forms.ComboBox

#Region "【メンバ変数】"
    Public Structure ComboItem

        Public Text As String   '表示テキスト   
        Public Code As String   '数値データ

        ''' <summary>
        ''' 
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Overrides Function ToString() As String
            Return Text
        End Function

        ''' <summary>
        ''' 
        ''' </summary>
        ''' <param name="tx"></param>
        ''' <param name="cd"></param>
        ''' <remarks></remarks>
        Public Sub New(ByVal tx As String, ByVal cd As Integer)
            Text = tx
            Code = cd
        End Sub

    End Structure

    Protected _item As ComboItem

    Private pBlnCencelEvent As Boolean

    ''' <summary>
    ''' 変更確認の有無
    ''' </summary>
    ''' <remarks></remarks>
    Private _chkEdit As Boolean

    ''' <summary>
    ''' フォーカス取得時の値
    ''' </summary>
    ''' <remarks></remarks>
    Private _val As String

    ''' <summary>
    ''' 一時保持用BackColor
    ''' </summary>
    ''' <remarks></remarks>
    Private _tmpBackColor As Color

    Private keyPressHandled As Boolean

    Private _ReadOnly As Boolean

#End Region

#Region "【コンストラクタ】"
    ''' <summary>
    ''' Default Constructor
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        MyBase.New()
        Me.BackColor = Color.White
        Me._tmpBackColor = Color.White
        pBlnCencelEvent = False
    End Sub
#End Region

#Region "【拡張プロパティ】"

#Region "[CheckEdit] - 変更確認の有無"
    ''' <summary>
    ''' 変更確認の有無
    ''' </summary>
    ''' <value>Boolean</value>
    ''' <returns>Boolean</returns>
    ''' <remarks>変更確認有無</remarks>
    <Category("動作")> _
    <Description("エディットコントロール変更時に、変更確認を行うかを指定します。")> _
    <Browsable(True)> _
    <DefaultValue(False)> _
    Public Property CheckEdit() As Boolean

        Get
            Return _chkEdit
        End Get

        Set(ByVal value As Boolean)
            _chkEdit = value
        End Set

    End Property
#End Region

#Region "[Code] - 選択コード取得・設定"
    ''' <summary>
    ''' 選択コード取得・設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Code() As String
        Get
            Return GetSelectedCode()
        End Get
        Set(value As String)
            SetSelectedCode(value)
        End Set
    End Property
#End Region

#Region "[CancelEvent] - イベントのキャンセルの設定を行います"
    ''' <summary>
    ''' イベントのキャンセルの設定を行います。
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("動作"), _
     Description("イベントのキャンセルの設定を行います"), _
     DefaultValue(False)> _
    Public Property CancelEvent() As Boolean
        Get
            Return pBlnCencelEvent
        End Get
        Set(value As Boolean)
            pBlnCencelEvent = value
        End Set
    End Property
#End Region

#Region "[ReadOnly] - エディット コントロールの中の文字列を変更できるかどうかを設定します"
    <Category("表示")> _
    <Description("エディット コントロールの中の文字列を変更できるかどうかを設定します。")> _
    <Browsable(True)> _
    <DefaultValue(False)> _
    Public Property [ReadOnly]() As Boolean
        Get
            Return Me._ReadOnly
        End Get

        Set(value As Boolean)
            Me._ReadOnly = value

            If value Then
                Me._tmpBackColor = Me.BackColor
                Me.BackColor = Color.FromArgb(255, 255, 192)
                Me.ContextMenu = New System.Windows.Forms.ContextMenu()
                'Me.SetStyle(ControlStyles.Selectable, True)
                Me.SetStyle(ControlStyles.UserMouse, True)
                Me.UpdateStyles()
            Else
                Me.BackColor = Me._tmpBackColor
                Me.ContextMenu = Nothing
                'Me.SetStyle(ControlStyles.Selectable, True)
                Me.SetStyle(ControlStyles.UserMouse, False)
                Me.UpdateStyles()
            End If
        End Set
    End Property
#End Region

#End Region

#Region "【拡張イベント】"

#Region "[OnGotFocus] - フォーカス取得拡張イベント"
    ''' <summary>
    ''' フォーカス取得時
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub OnGotFocus(ByVal e As System.EventArgs)
        'MyBase.OnGotFocus(e)
        'If Me.ReadOnly = False Then
        '    _tmpBackColor = Me.BackColor
        '    Me.BackColor = Color.FromArgb(204, 255, 255)
        'End If
    End Sub
#End Region

#Region "[OnEnter] - フォーカス取得拡張イベント"
    ''' <summary>
    ''' フォーカス取得
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub OnEnter(ByVal e As System.EventArgs)
        MyBase.OnGotFocus(e)

        '/// デザイン時は処理しない
        If DesignMode = True Then
            Exit Sub
        End If

        If Me.ReadOnly = False Then
            _tmpBackColor = Me.BackColor
            Me.BackColor = Color.FromArgb(204, 255, 255)
        End If

        MyBase.OnEnter(e)
        _val = Me.Text

    End Sub
#End Region

#Region "[OnLeave] - フォーカス喪失拡張イベント"
    ''' <summary>
    ''' フォーカス喪失時
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub OnLeave(ByVal e As System.EventArgs)
        MyBase.OnLeave(e)

        If pBlnCencelEvent = True Then
            pBlnCencelEvent = False
            Exit Sub
        End If

        If _val <> Me.Text And _chkEdit = True Then
            'Call SetEditFlg(True)
            gblnEditFlg = True
        End If

        Me.BackColor = _tmpBackColor

        'If Not PF_CheckValue() Then
        '    Me.Focus()
        '    Exit Sub
        'End If

    End Sub
#End Region

#Region "[OnValidating] - 入力値検査拡張イベント"
    Protected Overrides Sub OnValidating(e As System.ComponentModel.CancelEventArgs)
        If Not PF_CheckValue() Then
            e.Cancel = True
        End If
        MyBase.OnValidating(e)
    End Sub
#End Region

#Region "[OnKeyDown] - キー押下拡張イベント"
    ''' <summary>
    ''' キー押下時
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Protected Overrides Sub OnKeyDown(ByVal e As System.Windows.Forms.KeyEventArgs)
        If Not Me.ReadOnly Then
            MyBase.OnKeyDown(e)
            Select Case e.KeyCode
                Case Keys.Enter
                    If Me.Code = "" Then
                        Call MyBase.OnSelectedIndexChanged(e)
                    End If
                    SendKeys.Send("{TAB}")

            End Select
        Else
            Select Case e.KeyCode
                Case Keys.Left, Keys.Right, Keys.Home, Keys.End
                    keyPressHandled = True
                    Return
                Case Keys.C
                    If e.Control Then
                        keyPressHandled = True
                        Return
                    End If

                Case Keys.Enter
                    keyPressHandled = True
                    SendKeys.Send("{TAB}")
            End Select

            keyPressHandled = False
            e.Handled = True

            MyBase.OnKeyDown(e)
        End If
    End Sub
#End Region

#Region "[OnKeyPress] - キー押下拡張イベント"
    Protected Overrides Sub OnKeyPress(ByVal e As System.Windows.Forms.KeyPressEventArgs)
        If Me.ReadOnly Then
            If keyPressHandled Then
                e.Handled = False
            Else
                e.Handled = True
                Return
            End If
        End If

        MyBase.OnKeyPress(e)

    End Sub
#End Region

#Region "[OnTextChanged] - 値変更拡張イベント"
    Protected Overrides Sub OnTextChanged(e As System.EventArgs)

        Call UpdatePostion()

        MyBase.OnTextChanged(e)

    End Sub
#End Region

#Region "[OnMouseClick] - マウスクリック拡張イベント"
    Protected Overrides Sub OnMouseClick(e As System.Windows.Forms.MouseEventArgs)
        Call UpdatePostion()
        MyBase.OnMouseClick(e)
    End Sub
#End Region

#Region "[OnKeyUp] - キーアップ拡張イベント"
    Protected Overrides Sub OnKeyUp(e As System.Windows.Forms.KeyEventArgs)
        Call UpdatePostion()
        MyBase.OnKeyUp(e)
    End Sub
#End Region

#End Region

#Region "【メソッド】"

#Region "[ItemsAdd] - ComboBoxアイテム追加"
    ''' <summary>
    ''' ComboBoxアイテム追加
    ''' </summary>
    ''' <param name="strCd"></param>
    ''' <param name="strTx"></param>
    ''' <remarks></remarks>
    Public Sub ItemsAdd(ByVal strCd As String, ByVal strTx As String)
        Dim CmbItem As New ComboItem

        CmbItem.Code = strCd
        CmbItem.Text = strTx

        Me.Items.Add(CmbItem)

    End Sub
#End Region

#Region "[ItemsAdd] - ComboBoxアイテム追加(SQL結果)"
    ''' <summary>
    ''' ComboBoxアイテム追加(SQL結果)
    ''' </summary>
    ''' <param name="objRS">SQL結果</param>
    ''' <param name="Blank"></param>
    ''' <remarks></remarks>
    Public Sub ItemsAdd(ByVal objRS As clsRecordset, Optional ByVal Blank As Boolean = False)
        Dim CmbItem As New ComboItem

        If Blank Then
            CmbItem.Code = ""
            CmbItem.Text = ""
            Me.Items.Add(CmbItem)
        End If

        Do Until objRS.EOF
            CmbItem.Code = PF_Nz(objRS.Value(0))
            CmbItem.Text = PF_Nz(objRS.Value(1))

            Me.Items.Add(CmbItem)

            objRS.MoveNext()
        Loop
        objRS.MoveFirst()
    End Sub
#End Region

#Region "[ItemsAdd] - ComboBoxアイテム追加(SQL結果) 指定コード選択"
    ''' <summary>
    ''' ComboBoxアイテム追加(SQL結果)
    ''' </summary>
    ''' <param name="objRS"></param>
    ''' <param name="strCd"></param>
    ''' <param name="Blank"></param>
    ''' <remarks></remarks>
    Public Sub ItemsAdd(ByVal objRS As clsRecordset, strCd As String, Optional ByVal Blank As Boolean = False)
        Dim CmbItem As New ComboItem
        Dim idx As Int32
        Dim i As Int32

        If Blank Then
            CmbItem.Code = ""
            CmbItem.Text = ""
            Me.Items.Add(CmbItem)
        End If

        i = 0
        Do Until objRS.EOF

            If strCd = objRS.Value(0) Then
                idx = i
            End If

            CmbItem.Code = PF_Nz(objRS.Value(0))
            CmbItem.Text = PF_Nz(objRS.Value(1))

            Me.Items.Add(CmbItem)

            objRS.MoveNext()
            i = i + 1
        Loop
        Me.SelectedIndex = idx
    End Sub
#End Region

#Region "[GetSelectedCode] - 選択アイテムのコード取得"
    Private Function GetSelectedCode()
        Dim CmbItem As ComboBoxEx.ComboItem
        CmbItem = Me.SelectedItem
        Return CmbItem.Code
    End Function
#End Region

#Region "[SetSelectedCode] - 選択アイテムコード設定"
    Private Sub SetSelectedCode(strCode As String)
        Dim CmbItem As ComboBoxEx.ComboItem

        If String.IsNullOrEmpty(strCode) Then
            Exit Sub
        End If

        For i = 0 To Me.Items.Count - 1
            CmbItem = Me.Items(i)
            If CmbItem.Code = strCode Then
                Me.SelectedIndex = i
                Exit Sub
            End If
        Next
    End Sub
#End Region

#Region "[SetEditFlg] - フォームのblnEditFlgに [flg] を設定"
    ''' <summary>
    ''' フォームのblnEditFlgに [flg] を設定
    ''' </summary>
    ''' <param name="flg"></param>
    ''' <remarks></remarks>
    Public Sub SetEditFlg(ByVal flg As Boolean)

        Me._chkEdit = flg
        gblnEditFlg = True

    End Sub
#End Region

#Region "[PF_Nz] - Nz関数"
    Private Function PF_Nz(ByRef objValue As Object, Optional ByRef strNullChar As String = "") As String

        Try
            If IsDBNull(objValue) = False And IsNothing(objValue) = False Then
                Return CStr(objValue)
            Else
                Return strNullChar
            End If

        Catch ex As Exception

            Return strNullChar

        End Try

    End Function
#End Region

#Region "[UpdatePostion] - カレット位置更新"
    Private Sub UpdatePostion()

        Dim str As String = Me.Text
        'カレットの位置を取得
        Dim selectPos As Integer = Me.SelectionStart

        'カレットの位置までの行を数える
        Dim row As Integer = 1
        Dim startPos As Integer = 0
        Dim endPos As Integer
        While (True)
            endPos = str.IndexOf(vbLf, startPos)
            If (endPos < 0 Or endPos >= selectPos) Then
                Exit While
            End If
            startPos = endPos + 1
            row += 1
        End While

        '列の計算
        Dim col As Integer = selectPos - startPos + 1

        Formメイン.tssPosition.Text = row & "行、" & col & "列"

    End Sub
#End Region

#Region "[PF_CheckValue] - テキストがコンボアイテムにあるかチェックする"
    Private Function PF_CheckValue()
        Dim CmbItem As New ComboItem
        Dim blnFlg As Boolean = False
        Try
            If String.IsNullOrEmpty(Me.Text) Then
                Return True
            End If


            For i = 0 To Me.Items.Count - 1
                CmbItem = Me.Items(i)
                If StrConv(CmbItem.Text, VbStrConv.Uppercase) = StrConv(Me.Text, VbStrConv.Uppercase) Then
                    blnFlg = True
                    Exit For
                End If
            Next

            If blnFlg Then
                Me.Code = CmbItem.Code
            Else
                MessageBox.Show("指定した項目はリストにありません。", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return False
            End If

            Return True

        Catch ex As Exception
            Return False
        End Try
    End Function
#End Region

#End Region

End Class
