﻿Imports System.ComponentModel

Public Class LabelEX

    Inherits Label

    Dim blnAveText As Boolean
    Dim blnVertical As Boolean
    Dim clrBorder As Color

    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    Public Sub New()
        Me.AutoSize = False
        blnAveText = False
        blnVertical = False
        clrBorder = Color.Transparent
        Me.DoubleBuffered = True
    End Sub

    ''' <summary>
    ''' 境界線の色の取得、または設定を行う (FormBorderがNoneの時のみ)
    ''' </summary>
    <Category("武田")> _
    <Description("境界線の色の取得、または設定を行う (FormBorderがNoneの時のみ)")> _
    Public Property BorderColor() As Color
        Get
            Return clrBorder
        End Get
        Set(value As Color)
            clrBorder = value
        End Set
    End Property

    ''' <summary>
    ''' 均等割り付けの状態の取得、または設定を行う
    ''' </summary>
    <Category("武田")> _
    <Description("均等割り付けの状態の取得、または設定を行う")> _
    Public Property AverageText() As Boolean
        Get
            Return blnAveText
        End Get
        Set(value As Boolean)
            blnAveText = value
        End Set
    End Property

    ''' <summary>
    ''' 縦文字表示の状態の取得、または設定を行う
    ''' </summary>
    <Category("武田")> _
    <Description("縦文字表示の状態の取得、または設定を行う")> _
    Public Property VerticalText() As Boolean
        Get
            Return blnVertical
        End Get
        Set(value As Boolean)
            blnVertical = value
        End Set
    End Property

    ''' <summary>
    ''' Paintイベントハンドラ
    ''' </summary>
    ''' <param name="e">PaintEventArgs引数</param>
    Protected Overrides Sub OnPaint(e As System.Windows.Forms.PaintEventArgs)

        Dim intHeightAlign As Integer

        Dim intInterval As Single
        Dim intLeftDraw As Single
        Dim intTotalWidth As Single
        Dim intMaxWidth As Single

        intTotalWidth = 0
        intHeightAlign = 0
        intMaxWidth = 0
        intLeftDraw = 0

        '/// 境界線
        If MyBase.BorderStyle = Windows.Forms.BorderStyle.None And clrBorder <> Color.Transparent Then

            Dim rect As Rectangle = e.ClipRectangle
            rect.Width = rect.Width - 1
            rect.Height = rect.Height - 1

            e.Graphics.DrawRectangle(New Pen(New SolidBrush(clrBorder)), rect)

        End If


        '/// 縦書き
        If blnVertical = True And MyBase.Text <> "" Then

            intTotalWidth = (MyBase.Width / 2) - Math.Ceiling(GetStringWidth(e, Mid(MyBase.Text, 1, 1)) / 2) - 2

            For i As Integer = 1 To Len(MyBase.Text)
                intHeightAlign = intHeightAlign + GetStringHeight(e, Mid(MyBase.Text, i, 1))
            Next i

            intHeightAlign = (MyBase.Height / 2) - Math.Ceiling(intHeightAlign / 2)

            For i As Integer = 1 To Len(MyBase.Text)

                e.Graphics.DrawString(Mid(MyBase.Text, i, 1), MyBase.Font, New SolidBrush(MyBase.ForeColor), New Point(intTotalWidth, intHeightAlign))

                intHeightAlign = intHeightAlign + Math.Ceiling(GetStringHeight(e, Mid(MyBase.Text, i, 1)))

            Next i

            Exit Sub

        End If


        '/// 均等割り付けの設定
        If blnAveText = True And MyBase.Text <> "" Then

            If Len(MyBase.Text) > 1 Then

                ' 文字列の中で最大の場合を取得 (1文字毎に文字幅が異なるため均一にするために最大幅に合わせる)
                For i As Integer = 0 To Len(MyBase.Text) - 1

                    If intMaxWidth < GetStringWidth(e, Mid(MyBase.Text, i + 1, 1)) Then
                        intMaxWidth = GetStringWidth(e, Mid(MyBase.Text, i + 1, 1))
                    End If

                Next i

                intTotalWidth = Len(MyBase.Text) * intMaxWidth

                ' TextAlignがMiddleCenterの時は真ん中に表示
                If MyBase.TextAlign = ContentAlignment.MiddleCenter Then
                    intHeightAlign = (MyBase.Height / 2) - Math.Ceiling((GetStringHeight(e, MyBase.Text) / 2))
                End If

                ' 1文字間の長さを取得 (BorderStyleがFixedSingleの時は右端を-4引く)
                If MyBase.BorderStyle = Windows.Forms.BorderStyle.FixedSingle Or MyBase.BorderStyle = Windows.Forms.BorderStyle.Fixed3D Then
                    intInterval = (Me.Width - intTotalWidth - 5) / (Len(MyBase.Text) - 1)
                Else
                    intInterval = (Me.Width - intTotalWidth - 5) / (Len(MyBase.Text) - 1)
                    intLeftDraw = 2
                End If

                ' 指定位置に文字を描画
                For i As Integer = 1 To Len(MyBase.Text)

                    e.Graphics.DrawString(Mid(MyBase.Text, i, 1), MyBase.Font, New SolidBrush(MyBase.ForeColor), New Point(CInt(intLeftDraw), intHeightAlign))
                    intLeftDraw = intLeftDraw + intInterval + intMaxWidth

                Next

            Else

                MyBase.OnPaint(e)

            End If

        Else

            MyBase.OnPaint(e)

        End If

    End Sub

    ''' <summary>
    ''' 文字列の幅を取得する
    ''' </summary>
    ''' <param name="e">PaintEventArgs引数</param>
    ''' <param name="strText">対象文字列</param>
    ''' <returns>文字の幅</returns>
    Private Function GetStringWidth(ByVal e As System.Windows.Forms.PaintEventArgs, ByVal strText As String) As Single

        Dim characterRanges As CharacterRange() = {New CharacterRange(0, Len(strText))}
        Dim sf As New StringFormat

        sf.SetMeasurableCharacterRanges(characterRanges)

        '文字列のレイアウト四角形を指定する
        Dim layoutRect As New RectangleF(0, 0, MyBase.Width, MyBase.Height)

        '文字列に外接するRegion配列を取得する
        Dim stringRegions As Region() = e.Graphics.MeasureCharacterRanges(strText, MyBase.Font, layoutRect, sf)

        Return stringRegions(0).GetBounds(e.Graphics).Width

    End Function

    ''' <summary>
    ''' 文字列の高さを取得する
    ''' </summary>
    ''' <param name="e">PaintEventArgs引数</param>
    ''' <param name="strText">対象文字列</param>
    ''' <returns>文字の高さ</returns>
    Private Function GetStringHeight(ByVal e As System.Windows.Forms.PaintEventArgs, ByVal strText As String) As Single

        Dim characterRanges As CharacterRange() = {New CharacterRange(0, Len(strText))}
        Dim sf As New StringFormat

        sf.SetMeasurableCharacterRanges(characterRanges)

        '文字列のレイアウト四角形を指定する
        Dim layoutRect As New RectangleF(0, 0, MyBase.Width, MyBase.Height)

        '文字列に外接するRegion配列を取得する
        Dim stringRegions As Region() = e.Graphics.MeasureCharacterRanges(strText, MyBase.Font, layoutRect, sf)

        Return stringRegions(0).GetBounds(e.Graphics).Height

    End Function

End Class
