﻿Imports System.ComponentModel
Imports System.Windows.Forms

''' <summary>
''' 拡張テキストボックス
''' </summary>
''' <remarks>
''' 拡張テキストボックス
''' 　フォーカス取得時に背景を水色にする
''' 　ReadOnlyにすると背景が黄色になる
''' 　TextBoxTypeを[dateType]にすると日付の変換を行う
''' 　MaxLengthチェックを文字数かバイト数で選択できる
''' </remarks>
Public Class TextBoxEx
    Inherits System.Windows.Forms.TextBox

#Region "【定　数】"

    ''' <summary>
    ''' テキストボックスのタイプ列挙対
    ''' </summary>
    ''' <remarks></remarks>
    Public Enum TextBoxType As Integer
        textType = 1
        dateType = 2
        numberType = 3
        moneyType = 4
        yuubinType = 5
        nendoType = 6
    End Enum

#End Region

#Region "【メンバ変数】"

    ''' <summary>
    ''' テキストボックスのタイプ
    ''' </summary>
    ''' <remarks></remarks>
    Private _type As TextBoxType = TextBoxType.textType

    ''' <summary>
    ''' 変更確認の有無
    ''' </summary>
    ''' <remarks></remarks>
    Private _chkEdit As Boolean

    ''' <summary>
    ''' フォーカス取得時の値
    ''' </summary>
    ''' <remarks></remarks>
    Private _val As String

    ''' <summary>
    ''' 拡張テキストボックス内のエラー
    ''' </summary>
    ''' <remarks></remarks>
    Private _PrivateError As Boolean = False

    ''' <summary>
    ''' 一時保持用BackColor
    ''' </summary>
    ''' <remarks></remarks>
    Private _tmpBackColor As Color

    Private _byteCheck As Boolean

    Private _valMinValue As String
    Private _valMaxValue As String
    Private _moneyFoamat As String

#End Region

#Region "【コンストラクタ】"
    ''' <summary>
    ''' Default Constructor
    ''' </summary>
    Public Sub New()
        MyBase.New()
        Me.BackColor = Color.White
        Me._tmpBackColor = Color.White
    End Sub
#End Region

#Region "【拡張プロパティ】"

#Region "[InputType] - テキストボックスのタイプ"
    ''' <summary>
    ''' テキストボックスのタイプ
    ''' </summary>
    ''' <value>TextBoxType</value>
    ''' <returns>TextBoxType</returns>
    ''' <remarks></remarks>
    <Category("動作")> _
    <Description("エディットコントロールの入力タイプを指定します。")> _
    <Browsable(True)> _
    <DefaultValue(TextBoxType.textType), TypeConverter(GetType([Enum]))> _
    Public Property InputType() As TextBoxType

        Get
            Return _type
        End Get

        Set(ByVal value As TextBoxType)
            _type = value
        End Set

    End Property
#End Region

#Region "[CheckEdit] - 変更確認の有無"
    ''' <summary>
    ''' 変更確認の有無
    ''' </summary>
    ''' <value>Boolean</value>
    ''' <returns>Boolean</returns>
    ''' <remarks>変更確認有無</remarks>
    <Category("動作")> _
    <Description("エディットコントロール変更時に、変更確認を行うかを指定します。")> _
    <Browsable(True)> _
    <DefaultValue(False)> _
    Public Property CheckEdit() As Boolean

        Get
            Return _chkEdit
        End Get

        Set(ByVal value As Boolean)
            _chkEdit = value
        End Set

    End Property
#End Region

#Region "[MaxLengthByteCheck] - MaxLengthの判定を文字数で行うか、バイト数で行うかを設定する。"
    <Category("動作")> _
    <Description("MaxLengthの判定を文字数で行うか、バイト数で行うかを設定する。")> _
    <Browsable(True)> _
    <DefaultValue(False)> _
    Public Property MaxLengthByteCheck As Boolean
        Get
            Return _byteCheck
        End Get
        Set(value As Boolean)
            _byteCheck = value
        End Set
    End Property
#End Region

#Region "[MoneyFormat] - InputTypeが金額の時、設定した書式で表示を行う。"
    <Category("動作")> _
    <Description("InputTypeが金額の時、設定した書式で表示を行う。")> _
    <Browsable(True)> _
    <DefaultValue("")> _
    Public Property MoneyFormat As String
        Get
            Return _moneyFoamat
        End Get
        Set(value As String)
            _moneyFoamat = value
        End Set
    End Property
#End Region

#Region "[ValidationMinValue] - InputTypeが数値型の時、設定した値以下であれば警告表示を行う。"
    <Category("動作")> _
    <Description("InputTypeが数値型の時、設定した値以下であれば警告表示を行う。")> _
    <Browsable(True)> _
    <DefaultValue("")> _
    Public Property ValidationMinValue As String
        Get
            Return _valMinValue
        End Get
        Set(value As String)
            _valMinValue = value
        End Set
    End Property
#End Region

#Region "[ValidationMaxValue] - InputTypeが数値型の時、設定した値以上であれば警告表示を行う。"
    <Category("動作")> _
    <Description("InputTypeが数値型の時、設定した値以上であれば警告表示を行う。")> _
    <Browsable(True)> _
    <DefaultValue("")> _
    Public Property ValidationMaxValue As String
        Get
            Return _valMaxValue
        End Get
        Set(value As String)
            _valMaxValue = value
        End Set
    End Property
#End Region

#End Region

#Region "【拡張イベント】"

#Region "[OnGotFocus] - フォーカス取得拡張イベント"
    ''' <summary>
    ''' フォーカス取得拡張イベント
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' 背景色を水色[204,255,255]にする
    ''' 但し、[ReadOnly]プロパティが[True]の場合は何もしない。
    ''' </remarks>
    Protected Overrides Sub OnGotFocus(ByVal e As System.EventArgs)
        'MyBase.OnGotFocus(e)
        'If Me.ReadOnly = False Then
        '    _tmpBackColor = Me.BackColor
        '    Me.BackColor = Color.FromArgb(204, 255, 255)
        'End If
    End Sub
#End Region

#Region "[OnEnter] - フォーカス取得拡張イベント"
    ''' <summary>
    ''' フォーカス取得拡張イベント
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' [InputType]プロパティが[dateType]の場合、日付変換を行う[H25.01.01]→[4250101]
    ''' [InputType]プロパティが[moneyType]の場合、カンマ編集(除外)を行う。
    ''' [InputType]プロパティが[yuubinType]の場合、ハイフンの除外を行う。
    ''' [InputType]プロパティが[nendoType]の場合、日付変換を行う[H25]→[425]
    ''' </remarks>
    Protected Overrides Sub OnEnter(ByVal e As System.EventArgs)
        MyBase.OnEnter(e)

        Try
            MyBase.OnGotFocus(e)

            '/// デザイン時は処理しない
            If DesignMode = True Then
                Exit Sub
            End If

            If Me.ReadOnly = False Then
                _tmpBackColor = Me.BackColor
                Me.BackColor = Color.FromArgb(204, 255, 255)
            End If

            If Me.ReadOnly = False Then
                If String.IsNullOrEmpty(Me.Text) Then
                    Exit Sub
                End If

                Select Case _type
                    Case TextBoxType.textType
                        Exit Select

                    Case TextBoxType.dateType
                        GS_DateYMD_GotFocus()
                        Exit Select

                    Case TextBoxType.numberType
                        Exit Select

                    Case TextBoxType.moneyType
                        Me.Text = Replace(Me.Text, ",", "")
                        Exit Select

                    Case TextBoxType.yuubinType
                        Me.Text = Replace(Me.Text, "-", "")
                        Exit Select

                    Case TextBoxType.nendoType
                        GS_DateYMD_GotFocus()
                        Exit Select

                End Select

                '_val = Me.Text

            End If
        Catch ex As Exception
        Finally
            If Not _PrivateError Then
                _val = Me.Text
            End If
        End Try
    End Sub
#End Region

#Region "[OnValidating] - 入力値検査拡張イベント"
    ''' <summary>
    ''' 入力値検査拡張イベント
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' [CheckEdit]プロパティが[True]の場合、フォーカス取得時と値の比較をし、
    ''' 変更があれば配置元フォームの[blnEditFlg]に[True]を設定する。
    ''' [InputType]プロパティが[dateType]の場合、日付変換を行う[4250101]→[H25.01.01]
    ''' [InputType]プロパティが[numberType]の場合、数値チェックを行う。
    ''' [InputType]プロパティが[moneyType]の場合、数値チェックとカンマ編集を行う。
    ''' 但し、[ReadOnly]プロパティが[True]の場合は何もしない。
    ''' </remarks>
    Protected Overrides Sub OnValidating(ByVal e As System.ComponentModel.CancelEventArgs)
        MyBase.OnValidating(e)

        If Me.ReadOnly = False Then

            If _chkEdit Then
                If _val <> Me.Text Then
                    'Call SetEditFlg(True)
                End If
            End If

            _PrivateError = False
            If String.IsNullOrEmpty(Me.Text) Then
                Exit Sub
            End If

            Select Case _type
                Case TextBoxType.textType   '/// テキストタイプ
                    Exit Select

                Case TextBoxType.dateType   '/// 日付タイプ
                    If GF_DateYMD_BeforeUpdate() = False Then
                        e.Cancel = True
                        _PrivateError = True
                    Else
                        GS_DateYMD_LostFocus()
                    End If
                    Exit Select

                Case TextBoxType.numberType '/// 数値タイプ
                    If IsNumeric(Me.Text) = False Then
                        Call MessageBox.Show("数値以外の入力は出来ません。", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                        e.Cancel = True
                        _PrivateError = True
                    End If

                    If _valMinValue <> "" Then
                        If CType(_valMinValue, Double) >= CType(Me.Text, Double) Then
                            Dim v As Long = Me.Text
                            Call MessageBox.Show("入力した数値は認められていません。" & vbCrLf & _valMinValue & " 超過を入力してください。", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                            e.Cancel = True
                            _PrivateError = True
                        End If
                    End If

                    If _valMaxValue <> "" Then
                        If CType(_valMaxValue, Double) <= CType(Me.Text, Double) Then
                            Dim v As Long = Me.Text
                            Call MessageBox.Show("入力した数値は認められていません。" & vbCrLf & _valMaxValue & " 未満を入力してください。", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                            e.Cancel = True
                            _PrivateError = True
                        End If
                    End If

                    Exit Select

                Case TextBoxType.moneyType  '/// 金額タイプ
                    If IsNumeric(Me.Text) = False Then
                        Call MessageBox.Show("数値以外の入力は出来ません。", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                        e.Cancel = True
                        _PrivateError = True
                    Else
                        If _valMinValue <> "" Then
                            If CType(_valMinValue, Double) >= CType(Me.Text, Double) Then
                                Dim v As Long = Me.Text
                                Call MessageBox.Show("入力した数値は認められていません。" & vbCrLf & _valMinValue & " 超過を入力してください。", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                                e.Cancel = True
                                _PrivateError = True
                            End If
                        End If

                        If _valMaxValue <> "" Then
                            If CType(_valMaxValue, Double) <= CType(Me.Text, Double) Then
                                Dim v As Long = Me.Text
                                Call MessageBox.Show("入力した数値は認められていません。" & vbCrLf & _valMaxValue & " 未満を入力してください。", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                                e.Cancel = True
                                _PrivateError = True
                            End If
                        End If

                        Dim mf As String = "#,0.00"
                        If _moneyFoamat <> "" Then mf = _moneyFoamat

                        Try

                            Me.Text = Double.Parse(Replace(Me.Text, ",", "")).ToString(mf)

                        Catch ex As Exception
                            Call MessageBox.Show("入力した数値は認められていません。", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                            e.Cancel = True
                            _PrivateError = True
                        End Try

                    End If
                    Exit Select

                Case TextBoxType.yuubinType '/// 郵便番号タイプ
                    If IsNumeric(Replace(Me.Text, "-", "")) = False Then
                        Call MessageBox.Show("数値及び「-」以外の入力は出来ません。", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                        e.Cancel = True
                        _PrivateError = True
                    Else
                        If Replace(Me.Text, "-", "").Length = 7 Then
                            Me.Text = Mid(Replace(Me.Text, "-", ""), 1, 3) & "-" & Mid(Replace(Me.Text, "-", ""), 4, 4)
                        Else
                            Call MessageBox.Show("郵便番号は数値７桁かハイフン込み８桁で入力して下さい。", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                            e.Cancel = True
                            _PrivateError = True
                        End If
                    End If
                    Exit Select

                Case TextBoxType.nendoType  '/// 年度タイプ
                    If GF_Nendo_BeforeUpdate() = False Then
                        e.Cancel = True
                        _PrivateError = True
                    Else
                        GS_Nendo_LostFocus()
                    End If


            End Select
        End If

    End Sub
#End Region

#Region "[OnLeave] - フォーカス喪失時拡張イベント"
    ''' <summary>
    ''' フォーカス喪失時拡張イベント
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' 背景色をフォーカス取得前の色にする
    ''' 但し、[ReadOnly]プロパティが[True]の場合は何もしない。
    ''' </remarks>
    Protected Overrides Sub OnLeave(ByVal e As System.EventArgs)
        MyBase.OnLeave(e)

        If Me.ReadOnly = False Then
            Me.BackColor = _tmpBackColor

            If String.IsNullOrEmpty(Me.Text) Then
                Exit Sub
            End If

            Select Case _type
                Case TextBoxType.textType
                    Exit Select

                Case TextBoxType.dateType
                    Exit Select

                Case TextBoxType.numberType
                    Exit Select

                Case TextBoxType.moneyType
                    Exit Select

                Case TextBoxType.yuubinType
                    Exit Select

                Case TextBoxType.nendoType
                    Exit Select
            End Select

            If _chkEdit Then
                If _val <> Me.Text Then
                    gblnEditFlg = True
                End If
            End If
        End If
    End Sub
#End Region

#Region "[OnReadOnlyChanged] - 読取専用プロパティ変更拡張イベント"
    ''' <summary>
    ''' 読取専用プロパティ変更拡張イベント
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' [ReadOnly]プロパティが[True]の場合、背景色を薄い黄色[255,255,192]にする
    ''' [ReadOnly]プロパティが[false]の場合、背景色を白[255,255,255]にする
    ''' </remarks>
    Protected Overrides Sub OnReadOnlyChanged(ByVal e As System.EventArgs)
        MyBase.OnReadOnlyChanged(e)
        If Me.ReadOnly Then
            Me.BackColor = Color.FromArgb(255, 255, 192)
        Else
            Me.BackColor = Color.White
        End If
    End Sub
#End Region

#Region "[OnEnabledChanged] - 使用可否プロパティ変更拡張イベント"
    ''' <summary>
    ''' 使用可否プロパティ変更拡張イベント
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' [Enabled]が[True]で[ReadOnly]プロパティが[True]の場合、背景色を薄い黄色[255,255,192]にする
    ''' [Enabled]が[True]で[ReadOnly]プロパティが[false]の場合、背景色を白[255,255,255]にする
    ''' [Enabled]が[false]の場合、背景色をグレーにする
    ''' </remarks>
    Protected Overrides Sub OnEnabledChanged(e As System.EventArgs)
        MyBase.OnEnabledChanged(e)
        If Me.Enabled Then
            If Me.ReadOnly Then
                Me.BackColor = Color.FromArgb(255, 255, 192)
            Else
                Me.BackColor = Color.White
            End If
        Else
            Me.BackColor = Color.LightGray
        End If
    End Sub
#End Region

#Region "[OnTextChanged] - 値変更拡張イベント"
    ''' <summary>
    ''' 値変更拡張イベント
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>カレット位置更新</remarks>
    Protected Overrides Sub OnTextChanged(e As System.EventArgs)
        Call UpdatePostion()
        MyBase.OnTextChanged(e)
    End Sub
#End Region

#Region "[OnMouseClick] - マウスクリック拡張イベント"
    ''' <summary>
    ''' マウスクリック拡張イベント
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>カレット位置更新</remarks>
    Protected Overrides Sub OnMouseClick(e As System.Windows.Forms.MouseEventArgs)
        Call UpdatePostion()
        MyBase.OnMouseClick(e)
    End Sub
#End Region

#Region "[OnKeyUp] - キーアップ拡張イベント"
    ''' <summary>
    ''' キーアップ拡張イベント
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>カレット位置更新</remarks>
    Protected Overrides Sub OnKeyUp(e As System.Windows.Forms.KeyEventArgs)
        Call UpdatePostion()
        MyBase.OnKeyUp(e)
    End Sub
#End Region

#Region "[OnKeyDown] - キーダウン拡張イベント"
    ''' <summary>
    ''' キーダウン拡張イベント
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' [Enter]キー押下時に[TAB]キー押下時と同様の動きをする。
    ''' [Multiline]プロパティが[True]の場合は改行を行う。
    ''' [MultiLine]プロパティが[True]で[ReadOnly]プロパティが[True]の場合は[TAB]キー押下時と同様の動きをする
    ''' [Escape]キー押下時、フォーカス取得時の値に戻す。
    ''' [Ctrl]+[A]キー押下時、全選択を行う。
    ''' </remarks>
    Protected Overrides Sub OnKeyDown(ByVal e As System.Windows.Forms.KeyEventArgs)
        MyBase.OnKeyDown(e)

        Select Case e.KeyCode
            Case Keys.Enter
                '/// 単一行入力の場合はEnterキーでタブ移動
                If Me.Multiline = False Then
                    SendKeys.Send("{TAB}")
                Else
                    '/// 複数行入力の場合で読取専用の場合はEnterキーでタブ異動
                    If Me.ReadOnly = True Then
                        SendKeys.Send("{TAB}")
                    End If
                End If

            Case Keys.Escape
                '/// Escapeキーでフォーカス取得時の値に戻す
                Me.Text = _val

            Case Keys.A
                '/// Ctrl+Aで全選択
                If e.Control Then
                    Me.SelectAll()
                End If

        End Select
    End Sub
#End Region

#Region "[WndProc] - 拡張ウィンドウプロシージャ"
    Protected Overrides Sub WndProc(ByRef m As System.Windows.Forms.Message)
        Const WM_CHAR As Integer = &H102
        Const WM_PASTE As Integer = &H302

        Select Case m.Msg
            Case WM_CHAR
                Dim eKeyPress As New KeyPressEventArgs(Microsoft.VisualBasic.ChrW(m.WParam.ToInt32()))
                Me.OnChar(eKeyPress)

                If eKeyPress.Handled Then
                    Return
                End If

            Case WM_PASTE
                Me.OnPaste(New System.EventArgs())
                Return
        End Select

        MyBase.WndProc(m)
    End Sub
#End Region

#Region "[OnChar] - キー入力拡張イベント "
    ''' <summary>
    ''' キー入力拡張イベント
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' [MaxLengthByteCheck]プロパティが[True]の場合、[MaxLength]プロパティの値とバイト数で比較する。
    ''' [MaxLengthByteCheck]プロパティが[False]の場合、[MaxLength]プロパティの値と文字数で比較する。
    ''' </remarks>
    Protected Overridable Sub OnChar(ByVal e As System.Windows.Forms.KeyPressEventArgs)
        '/// 制御文字の場合
        If Char.IsControl(e.KeyChar) Then
            Select Case e.KeyChar
                Case Microsoft.VisualBasic.ChrW(Keys.Enter)
                    '/// Enterキーの場合は文字数チェックを行う
                Case Else
                    '/// Enterキー以外の場合は何もしない
                    Return
            End Select
        End If

        Dim sjisEncoding As System.Text.Encoding = System.Text.Encoding.GetEncoding("Shift_JIS")
        Dim textByteCount As Integer = sjisEncoding.GetByteCount(Me.Text)
        Dim inputByteCount As Integer = sjisEncoding.GetByteCount(e.KeyChar.ToString())
        Dim selectedTextByteCount As Integer = sjisEncoding.GetByteCount(Me.SelectedText)

        Dim textCount As Integer = Me.Text.Length
        Dim inputCount As Integer = e.KeyChar.ToString().Length
        Dim selectedTextCount As Integer = Me.SelectedText.Length


        If _byteCheck Then
            If textByteCount + inputByteCount - selectedTextByteCount > Me.MaxLength Then
                e.Handled = True
            End If
        Else
            If textCount + inputCount - selectedTextCount > Me.MaxLength Then
                e.Handled = True
            End If
        End If
    End Sub
#End Region

#Region "[OnPaste] - ペースト拡張イベント"
    ''' <summary>
    ''' ペースト拡張イベント
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks>
    ''' [MaxLengthByteCheck]プロパティが[True]の場合、[MaxLength]プロパティの値とバイト数で比較する。
    ''' [MaxLengthByteCheck]プロパティが[False]の場合、[MaxLength]プロパティの値と文字数で比較する。
    ''' </remarks>
    Protected Overridable Sub OnPaste(ByVal e As System.EventArgs)
        Dim clipboardText As Object = Clipboard.GetDataObject().GetData(System.Windows.Forms.DataFormats.Text)

        If clipboardText Is Nothing Then
            Return
        End If

        Dim sjisEncoding As System.Text.Encoding = System.Text.Encoding.GetEncoding("Shift_JIS")
        Dim inputText As String = clipboardText.ToString()
        Dim textByteCount As Integer = sjisEncoding.GetByteCount(Me.Text)
        Dim inputByteCount As Integer = sjisEncoding.GetByteCount(inputText)
        Dim selectedTextByteCount As Integer = sjisEncoding.GetByteCount(Me.SelectedText)
        Dim remainByteCount As Integer = Me.MaxLength - (textByteCount - selectedTextByteCount)

        Dim textCount As Integer = Me.Text.Length
        Dim inputCount As Integer = inputText.Length
        Dim selectedTextCount As Integer = Me.SelectedText.Length
        Dim remainCount As Integer = Me.MaxLength - (textCount - selectedTextCount)

        If _byteCheck Then
            If remainByteCount <= 0 Then
                Return
            End If

            If remainByteCount >= inputByteCount Then
                Me.SelectedText = inputText
            Else
                Dim strTmp As String = ""
                For i = 0 To inputText.Length - 1
                    If sjisEncoding.GetByteCount(strTmp & inputText.Substring(0, 1)) > remainByteCount Then
                        Exit For
                    End If
                    strTmp = strTmp & inputText.Substring(0, 1)
                    inputText = inputText.Substring(1)
                Next
                Me.SelectedText = strTmp
            End If
        Else
            If remainCount <= 0 Then
                Return
            End If

            If remainCount >= inputCount Then
                Me.SelectedText = inputText
            Else
                Me.SelectedText = inputText.Substring(0, remainCount)
            End If
        End If

    End Sub
#End Region

#End Region

#Region "【メソッド】"

#Region "[GS_DateYMD_GotFocus] - 日付入力コントロールのフォーカス取得時処理"
    ''' <summary>
    ''' 日付入力コントロールのフォーカス取得時処理
    ''' </summary>
    ''' <remarks>
    ''' 年号を数値に変換する
    ''' [M]→[1]
    ''' [T]→[2]
    ''' [S]→[3]
    ''' [H]→[4]
    ''' [.]を除外する
    ''' </remarks>
    Public Sub GS_DateYMD_GotFocus()

        On Error Resume Next

        Dim strValue As String

        strValue = Me.Text

        If strValue Is Nothing Then
            Exit Sub
        End If

        strValue = Replace(strValue, "M", "1")
        strValue = Replace(strValue, "T", "2")
        strValue = Replace(strValue, "S", "3")
        strValue = Replace(strValue, "H", "4")
        strValue = Replace(strValue, ".", "")

        'strValue = Replace(strValue, "明治", "1")
        'strValue = Replace(strValue, "大正", "2")
        'strValue = Replace(strValue, "昭和", "3")
        'strValue = Replace(strValue, "平成", "4")
        'strValue = Replace(Replace(Replace(Replace(strValue, "年", ""), "月", ""), "日", ""), " ", "")

        Me.Text = strValue

    End Sub
#End Region

#Region "[GS_DateYMD_LostFocus] - 日付入力コントロールのフォーカス喪失時処理"
    ''' <summary>
    ''' 日付入力コントロールのフォーカス喪失時処理
    ''' </summary>
    ''' <remarks>
    ''' 先頭の数値を年号に変換する
    ''' [1]→[M]
    ''' [2]→[T]
    ''' [3]→[S]
    ''' [4]→[H]
    ''' [.]編集を行う
    ''' </remarks>
    Public Sub GS_DateYMD_LostFocus()

        Dim strValue As String

        On Error Resume Next
        strValue = Me.Text

        If strValue Is Nothing Then
            Exit Sub
        End If

        strValue = Replace(Replace(strValue, "/", ""), ".", "")

        Select Case Mid(strValue, 1, 1).ToUpper

            Case "1", "M" : strValue = "M" & Mid(strValue, 2, 2) & "." & Mid(strValue, 4, 2) & "." & Mid(strValue, 6, 2)
            Case "2", "T" : strValue = "T" & Mid(strValue, 2, 2) & "." & Mid(strValue, 4, 2) & "." & Mid(strValue, 6, 2)
            Case "3", "S" : strValue = "S" & Mid(strValue, 2, 2) & "." & Mid(strValue, 4, 2) & "." & Mid(strValue, 6, 2)
            Case "4", "H" : strValue = "H" & Mid(strValue, 2, 2) & "." & Mid(strValue, 4, 2) & "." & Mid(strValue, 6, 2)
            Case Else : strValue = ""

        End Select

        'Select Case Mid(strValue, 1, 1)

        '    Case "1", "明治" : strValue = "明治" & Mid(strValue, 2, 2) & "年" & Mid(strValue, 4, 2) & "月" & Mid(strValue, 6, 2) & "日"
        '    Case "2", "大正" : strValue = "大正" & Mid(strValue, 2, 2) & "年" & Mid(strValue, 4, 2) & "月" & Mid(strValue, 6, 2) & "日"
        '    Case "3", "昭和" : strValue = "昭和" & Mid(strValue, 2, 2) & "年" & Mid(strValue, 4, 2) & "月" & Mid(strValue, 6, 2) & "日"
        '    Case "4", "平成" : strValue = "平成" & Mid(strValue, 2, 2) & "年" & Mid(strValue, 4, 2) & "月" & Mid(strValue, 6, 2) & "日"
        '    Case Else : strValue = ""

        'End Select

        Me.Text = strValue

    End Sub
#End Region

#Region "[GF_DateYMD_BeforeUpdate] - 日付入力コントロールの更新前処理"
    ''' <summary>
    ''' 日付入力コントロールの更新前処理
    ''' </summary>
    ''' <returns>正常終了時:True / 異常時:False</returns>
    ''' <remarks>
    ''' 桁数チェック、日付の妥当性チェックを行う。
    ''' </remarks>
    Public Function GF_DateYMD_BeforeUpdate() As Boolean

        Dim strValue As String

        strValue = Me.Text

        If String.IsNullOrEmpty(strValue) Then
            Return True
        End If

        '文字数チェック
        If Len(strValue) <> 7 Then

            Call MessageBox.Show("年月日の入力は以下の形式で行ってください。" & vbCrLf & vbCrLf & _
                        "  １    桁目：明治⇒1　大正⇒2　昭和⇒3　平成⇒4" & vbCrLf & _
                        "  ２～３桁目：年を２桁の数字" & vbCrLf & _
                        "  ４～５桁目：月を２桁の数字" & vbCrLf & _
                        "  ６～７桁目：日を２桁の数字" & vbCrLf & vbCrLf & _
                        "  【例】『H17.04.01』と入力したい場合は『4170401』と入力。 ", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

            Return False

        End If

        strValue = Replace(Replace(strValue, "/", ""), ".", "")

        Select Case Mid(strValue, 1, 1).ToUpper

            Case "1", "M" : strValue = "M" & Mid(strValue, 2, 2) & "." & Mid(strValue, 4, 2) & "." & Mid(strValue, 6, 2)
            Case "2", "T" : strValue = "T" & Mid(strValue, 2, 2) & "." & Mid(strValue, 4, 2) & "." & Mid(strValue, 6, 2)
            Case "3", "S" : strValue = "S" & Mid(strValue, 2, 2) & "." & Mid(strValue, 4, 2) & "." & Mid(strValue, 6, 2)
            Case "4", "H" : strValue = "H" & Mid(strValue, 2, 2) & "." & Mid(strValue, 4, 2) & "." & Mid(strValue, 6, 2)
            Case Else : strValue = ""

        End Select

        '日付チェック
        If IsDate(Replace(strValue, ".", "/")) = False Then

            Call MessageBox.Show("年月日の入力は以下の形式で行ってください。" & vbCrLf & vbCrLf & _
                        "  １    桁目：明治⇒1　大正⇒2　昭和⇒3　平成⇒4" & vbCrLf & _
                        "  ２～３桁目：年を２桁の数字" & vbCrLf & _
                        "  ４～５桁目：月を２桁の数字" & vbCrLf & _
                        "  ６～７桁目：日を２桁の数字" & vbCrLf & vbCrLf & _
                        "  【例】『H17.04.01』と入力したい場合は『4170401』と入力。 ", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

            Return False

        End If

        Return True

    End Function
#End Region

#Region "[GS_Nendo_LostFocus] - 年度入力コントロールのフォーカス喪失時処理"
    ''' <summary>
    ''' 年度入力コントロールのフォーカス喪失時処理
    ''' </summary>
    ''' <remarks>
    ''' 先頭の数値を年号に変換する
    ''' [1]→[M]
    ''' [2]→[T]
    ''' [3]→[S]
    ''' [4]→[H]
    ''' </remarks>
    Public Sub GS_Nendo_LostFocus()

        Dim strValue As String

        On Error Resume Next
        strValue = Me.Text

        If strValue Is Nothing Then
            Exit Sub
        End If

        strValue = Replace(Replace(strValue, "/", ""), ".", "")

        Select Case Mid(strValue, 1, 1).ToUpper

            Case "1", "M" : strValue = "M" & Mid(strValue, 2, 2)
            Case "2", "T" : strValue = "T" & Mid(strValue, 2, 2)
            Case "3", "S" : strValue = "S" & Mid(strValue, 2, 2)
            Case "4", "H" : strValue = "H" & Mid(strValue, 2, 2)
            Case Else : strValue = ""

        End Select

        Me.Text = strValue

    End Sub
#End Region

#Region "[GF_Nendo_BeforeUpdate] - 年度入力コントロールの更新前処理"
    ''' <summary>
    ''' 年度入力コントロールの更新前処理
    ''' </summary>
    ''' <returns>正常終了時:True / 異常時:False</returns>
    ''' <remarks>
    ''' 桁数チェック、日付の妥当性チェックを行う。
    ''' </remarks>
    Public Function GF_Nendo_BeforeUpdate() As Boolean

        Dim strValue As String

        strValue = Me.Text

        If String.IsNullOrEmpty(strValue) Then
            Return True
        End If

        '文字数チェック
        If Len(strValue) <> 3 Then

            Call MessageBox.Show("年月日の入力は以下の形式で行ってください。" & vbCrLf & vbCrLf & _
                        "  １    桁目：明治⇒1　大正⇒2　昭和⇒3　平成⇒4" & vbCrLf & _
                        "  ２～３桁目：年を２桁の数字" & vbCrLf & _
                        "  【例】『H25』と入力したい場合は『425』と入力。 ", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

            Return False

        End If

        strValue = Replace(Replace(strValue, "/", ""), ".", "")

        Select Case Mid(strValue, 1, 1).ToUpper

            Case "1", "M" : strValue = "M" & Mid(strValue, 2, 2)
            Case "2", "T" : strValue = "T" & Mid(strValue, 2, 2)
            Case "3", "S" : strValue = "S" & Mid(strValue, 2, 2)
            Case "4", "H" : strValue = "H" & Mid(strValue, 2, 2)
            Case Else : strValue = ""

        End Select

        '日付チェック
        If IsDate(strValue & "/04/01") = False Then

            Call MessageBox.Show("年月日の入力は以下の形式で行ってください。" & vbCrLf & vbCrLf & _
                        "  １    桁目：明治⇒1　大正⇒2　昭和⇒3　平成⇒4" & vbCrLf & _
                        "  ２～３桁目：年を２桁の数字" & vbCrLf & _
                        "  【例】『H25』と入力したい場合は『425』と入力。 ", My.Application.Info.AssemblyName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)

            Return False

        End If

        Return True

    End Function
#End Region

#Region "★公有財産では未使用のため廃止 - [SetEditFlg] - フォームのblnEditFlgに [flg] を設定"
    ' ''' <summary>
    ' ''' フォームのblnEditFlgに [flg] を設定
    ' ''' </summary>
    ' ''' <param name="flg"></param>
    ' ''' <remarks></remarks>
    'Public Sub SetEditFlg(ByVal flg As Boolean)
    '    Dim frm As frmBase

    '    frm = Me.FindForm()
    '    frm.blnEditFlg = flg

    'End Sub
#End Region

#Region "[UpdatePostion] - カレット位置更新"
    ''' <summary>
    ''' カレット位置更新
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub UpdatePostion()

        Dim str As String = Me.Text
        'カレットの位置を取得
        Dim selectPos As Integer = Me.SelectionStart

        'カレットの位置までの行を数える
        Dim row As Integer = 1
        Dim startPos As Integer = 0
        Dim endPos As Integer
        Dim col As Integer

        'テキストの配置位置によって列カレットを変更
        'If Me.TextAlign = HorizontalAlignment.Right Then

        '    '///右側表示

        '    endPos = 0

        '    While (True)
        '        startPos = str.IndexOf(vbLf, startPos)
        '        If (endPos > 0 Or endPos <= selectPos) Then
        '            Exit While
        '        End If
        '        startPos = endPos - 1
        '        row += 1
        '    End While

        '    '列の計算
        '    col = selectPos - startPos - 1


        'Else

        '/// 左側、中央表示

        While (True)
            endPos = str.IndexOf(vbLf, startPos)
            If (endPos < 0 Or endPos >= selectPos) Then
                Exit While
            End If
            startPos = endPos + 1
            row += 1
        End While

        '列の計算
        col = selectPos - startPos + 1

        'End If

        Formメイン.tssPosition.Text = row & "行、" & col & "列"

    End Sub
#End Region

#End Region

End Class

